#!/usr/bin/perl -w

BEGIN
{
	chdir 't' if -d 't';
	use lib '../lib', '../blib/lib';
}

use strict;

use Test::More tests => 25;

use Test::Exception;
use Test::MockObject;

my $mock_out = Test::MockObject->new();
$mock_out->fake_module( 'Jellybean::Response', new => sub { $mock_out } );

my $module = 'Jellybean::Engine';
use_ok( $module ) or exit;
can_ok( $module, 'new' );

my $e = Jellybean::Engine->new();
isa_ok( $e, $module );

can_ok( $module, 'add_server' );
throws_ok { $e->add_server() }
	qr/No server provided/,     'add_server() should carp with no server given';
throws_ok { $e->add_server( 1 ) }
	qr/Invalid server/,         '... or if server looks invalid';

my $mock_server = Test::MockObject->new();

# and it mocks a Jellybean::Server
{
	package Test::MockObject;
	Class::ActsLike->import( 'Jellybean::Server' );
}

lives_ok { $e->add_server( $mock_server ) } '... but not if server is okay';

can_ok( $e, 'servers' );
is_deeply( $e->servers(), [ $mock_server ],
	'servers() should return installed servers' );

my $ms2 = Test::MockObject->new();
$e->add_server( $ms2 );
is_deeply( $e->servers(), [ $mock_server, $ms2 ], '... in added order' );

$mock_server->set_false( 'incoming' );
$ms2->set_true( 'incoming' )
	->set_always( request => 'request' );

can_ok( $e, 'listen' );
my @results = $e->listen();
is( $mock_server->next_call(), 'incoming',
	'listen() should check for incoming connections' );
is( $ms2->next_call(), 'incoming',
	'... on all installed servers' );
is( $ms2->next_call(), 'request',
	'... fetching request from active connections' );
is( @results, 1, '... returning results for each active connection' );
is_deeply( $results[0], [ 'request', $ms2 ],
	'... as request and server in array ref' );

can_ok( $e, 'add_action' );
my $mock = Test::MockObject->new();
$mock->set_true( 'activate' );
$mock_server->set_true( 'respond' );
$e->add_action( 'default', $mock );

can_ok( $e, 'process_request' );
require Jellybean::Request;
my $req = Jellybean::Request->new( conn => '1', path => [qw( my path )] );
$mock_out->set_true( 'connection' );

$e->process_request( $req, $mock_server );
my ($method, $args) = $mock->next_call();
is( $method, 'activate',
	'process_request() should activate default action lacking better option' );
is( $args->[1], $req,      '... passing the request' );
is( $args->[2], $mock_out, '... and a new response object' );

is( $mock_out->next_call(),
	'connection',          '... grabbing request connection' );

($method, $args) = $mock_server->next_call();
is( $method, 'respond', '... telling the server to respond' );

$e->add_action( 'fear', $mock );
$req = Jellybean::Request->new( conn => '1', path => [qw( fear my path )] );
$e->process_request( $req, $mock_server );
($method, $args) = $mock->next_call();
is( $method, 'activate', '... processing request for named action if found' );
is_deeply( [ $req->path() ], [qw( my path )], '... stripping off name' );

__END__
